@@ -118,6 +118,16 @@ test_that("Checks input", {
 
 })
 
+test_that("fullmatch warns when given a 'within' arg that it's going to ignore", {
+    m <- matrix(1, nrow = 2, ncol = 3,
+                dimnames = list(c("a", "b"), c('d', 'e', 'f')))
+    B <- rep(1:3, each = 2)
+    names(B) <- letters[1:6]
+    em <- exactMatch(B, rep(c(0,1), 3))
+    expect_warning(fullmatch(m, within=em), "gnor")
+    expect_warning(fullmatch(as.InfinitySparseMatrix(m), within=em), "gnor")
+})
+
 test_that("Reversion Test: Fullmatch handles omit.fraction for matrices", {
   # this bug was discovered while working on pairmatch, but it would appear to be
   # a fullmatch bug, though it might actually be in in subdivstrat or fmatch.
@@ -382,3 +392,16 @@ test_that("fullmatch UI cleanup", {
 
 
 })
+
+test_that("NAs in irrelevant data slots don't trip us up", {
+  n <- 16
+  Z <- c(rep(0, n/2), rep(1, n/2))
+  X1 <- rep(c(1,2,3,4), each = n/4)
+  B <- rep(c(0,1), n/2)
+  B[1] <- NA
+  test.data <- data.frame(Z, X1, B)
+  rm(Z)
+  rm(X1)
+  rm(B)
+  expect_equal(length(fullmatch(Z~X1, data=test.data)), n)
+})
